@@ -17,6 +17,10 @@
 #include <string_view>
 #endif
 
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+#include <atomic>
+#endif
+
 #include <cassert>
 
 namespace toml
@@ -48,13 +52,18 @@ template<typename TC>
 error_info make_type_error(const basic_value<TC>&, const std::string&, const value_t);
 
 template<typename TC>
-error_info make_not_found_error(const basic_value<TC>&, const std::string&, const std::string&);
+error_info make_not_found_error(const basic_value<TC>&, const std::string&, const typename basic_value<TC>::key_type&);
 
 template<typename TC>
 void change_region_of_value(basic_value<TC>&, const basic_value<TC>&);
 
 template<typename TC, value_t V>
 struct getter;
+
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+template<typename TC>
+void unset_access_flag(basic_value<TC>&);
+#endif
 } // detail
 
 template<typename TypeConfig>
@@ -76,6 +85,7 @@ class basic_value
     using array_type           = typename config_type::template array_type<value_type>;
     using table_type           = typename config_type::template table_type<key_type, value_type>;
     using comment_type         = typename config_type::comment_type;
+    using char_type            = typename string_type::value_type;
 
   private:
 
@@ -85,6 +95,9 @@ class basic_value
 
     basic_value() noexcept
         : type_(value_t::empty), empty_('\0'), region_{}, comments_{}
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     ~basic_value() noexcept {this->cleanup();}
 
@@ -92,6 +105,9 @@ class basic_value
 
     basic_value(const basic_value& v)
         : type_(v.type_), region_(v.region_), comments_(v.comments_)
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{v.accessed()}
+#endif
     {
         switch(this->type_)
         {
@@ -111,6 +127,9 @@ class basic_value
     basic_value(basic_value&& v)
         : type_(v.type()), region_(std::move(v.region_)),
           comments_(std::move(v.comments_))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{v.accessed()}
+#endif
     {
         switch(this->type_)
         {
@@ -136,6 +155,9 @@ class basic_value
         this->type_     = v.type_;
         this->region_   = v.region_;
         this->comments_ = v.comments_;
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = v.accessed();
+#endif
         switch(this->type_)
         {
             case value_t::boolean        : assigner(boolean_        , v.boolean_        ); break;
@@ -160,6 +182,9 @@ class basic_value
         this->type_     = v.type_;
         this->region_   = std::move(v.region_);
         this->comments_ = std::move(v.comments_);
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = v.accessed();
+#endif
         switch(this->type_)
         {
             case value_t::boolean        : assigner(boolean_        , std::move(v.boolean_        )); break;
@@ -183,6 +208,9 @@ class basic_value
     basic_value(basic_value v, std::vector<std::string> com)
         : type_(v.type()), region_(std::move(v.region_)),
           comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{v.accessed()}
+#endif
     {
         switch(this->type_)
         {
@@ -208,6 +236,9 @@ class basic_value
         : type_(other.type_),
           region_(std::move(other.region_)),
           comments_(std::move(other.comments_))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{other.accessed()}
+#endif
     {
         switch(other.type_)
         {
@@ -253,6 +284,9 @@ class basic_value
         : type_(other.type_),
           region_(std::move(other.region_)),
           comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{other.accessed()}
+#endif
     {
         switch(other.type_)
         {
@@ -299,6 +333,10 @@ class basic_value
         this->region_ = other.region_;
         this->comments_    = comment_type(other.comments_);
         this->type_        = other.type_;
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = other.accessed();
+#endif
+
         switch(other.type_)
         {
             // use auto-convert in constructor
@@ -358,6 +396,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::boolean), boolean_(boolean_storage(x, fmt)),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(boolean_type x)
     {
@@ -369,6 +410,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::boolean;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->boolean_, boolean_storage(x, fmt));
         return *this;
     }
@@ -392,6 +436,9 @@ class basic_value
     basic_value(integer_type x, integer_format_info fmt, std::vector<std::string> com, region_type reg)
         : type_(value_t::integer), integer_(integer_storage(std::move(x), std::move(fmt))),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(integer_type x)
     {
@@ -403,6 +450,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::integer;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->integer_, integer_storage(std::move(x), std::move(fmt)));
         return *this;
     }
@@ -438,6 +488,9 @@ class basic_value
     basic_value(T x, integer_format_info fmt, std::vector<std::string> com, region_type reg)
         : type_(value_t::integer), integer_(integer_storage(std::move(x), std::move(fmt))),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     template<typename T, enable_if_integer_like_t<T> = nullptr>
     basic_value& operator=(T x)
@@ -450,6 +503,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::integer;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->integer_, integer_storage(x, std::move(fmt)));
         return *this;
     }
@@ -473,6 +529,9 @@ class basic_value
     basic_value(floating_type x, floating_format_info fmt, std::vector<std::string> com, region_type reg)
         : type_(value_t::floating), floating_(floating_storage(std::move(x), std::move(fmt))),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(floating_type x)
     {
@@ -484,6 +543,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::floating;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->floating_, floating_storage(std::move(x), std::move(fmt)));
         return *this;
     }
@@ -522,6 +584,9 @@ class basic_value
     basic_value(T x, floating_format_info fmt, std::vector<std::string> com, region_type reg)
         : type_(value_t::floating), floating_(floating_storage(x, std::move(fmt))),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
 
     template<typename T, enable_if_floating_like_t<T> = nullptr>
@@ -535,6 +600,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::floating;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->floating_, floating_storage(x, std::move(fmt)));
         return *this;
     }
@@ -559,6 +627,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::string), string_(string_storage(std::move(x), std::move(fmt))),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(string_type x)
     {
@@ -570,6 +641,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::string;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->string_, string_storage(x, std::move(fmt)));
         return *this;
     }
@@ -592,6 +666,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::string), string_(string_storage(string_type(x), std::move(fmt))),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(const typename string_type::value_type* x)
     {
@@ -603,6 +680,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::string;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->string_, string_storage(string_type(x), std::move(fmt)));
         return *this;
     }
@@ -627,6 +707,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::string), string_(string_storage(string_type(x), std::move(fmt))),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(string_view_type x)
     {
@@ -638,11 +721,77 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::string;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->string_, string_storage(string_type(x), std::move(fmt)));
         return *this;
     }
 
 #endif // TOML11_HAS_STRING_VIEW
+
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            cxx::negation<std::is_same<cxx::remove_cvref_t<T>, string_type>>,
+            detail::is_1byte_std_basic_string<T>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value(const T& x)
+        : basic_value(x, string_format_info{}, std::vector<std::string>{}, region_type{})
+    {}
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            cxx::negation<std::is_same<cxx::remove_cvref_t<T>, string_type>>,
+            detail::is_1byte_std_basic_string<T>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value(const T& x, string_format_info fmt)
+        : basic_value(x, std::move(fmt), std::vector<std::string>{}, region_type{})
+    {}
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            cxx::negation<std::is_same<cxx::remove_cvref_t<T>, string_type>>,
+            detail::is_1byte_std_basic_string<T>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value(const T& x, std::vector<std::string> com)
+        : basic_value(x, string_format_info{}, std::move(com), region_type{})
+    {}
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            cxx::negation<std::is_same<cxx::remove_cvref_t<T>, string_type>>,
+            detail::is_1byte_std_basic_string<T>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value(const T& x, string_format_info fmt, std::vector<std::string> com)
+        : basic_value(x, std::move(fmt), std::move(com), region_type{})
+    {}
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            cxx::negation<std::is_same<cxx::remove_cvref_t<T>, string_type>>,
+            detail::is_1byte_std_basic_string<T>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value(const T& x, string_format_info fmt,
+                std::vector<std::string> com, region_type reg)
+        : type_(value_t::string),
+          string_(string_storage(detail::string_conv<string_type>(x), std::move(fmt))),
+          region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
+    {}
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            cxx::negation<std::is_same<cxx::remove_cvref_t<T>, string_type>>,
+            detail::is_1byte_std_basic_string<T>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value& operator=(const T& x)
+    {
+        string_format_info fmt;
+        if(this->is_string())
+        {
+            fmt = this->as_string_fmt();
+        }
+        this->cleanup();
+        this->type_   = value_t::string;
+        this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
+        assigner(this->string_, string_storage(detail::string_conv<string_type>(x), std::move(fmt)));
+        return *this;
+    }
+
     // }}}
 
     // constructor (local_date) =========================================== {{{
@@ -663,6 +812,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::local_date), local_date_(local_date_storage(x, fmt)),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(local_date_type x)
     {
@@ -674,6 +826,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::local_date;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->local_date_, local_date_storage(x, fmt));
         return *this;
     }
@@ -698,6 +853,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::local_time), local_time_(local_time_storage(x, fmt)),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(local_time_type x)
     {
@@ -709,6 +867,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::local_time;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->local_time_, local_time_storage(x, fmt));
         return *this;
     }
@@ -746,6 +907,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::local_time;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->local_time_, local_time_storage(local_time_type(x), std::move(fmt)));
         return *this;
     }
@@ -770,6 +934,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::local_datetime), local_datetime_(local_datetime_storage(x, fmt)),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(local_datetime_type x)
     {
@@ -781,6 +948,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::local_datetime;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->local_datetime_, local_datetime_storage(x, fmt));
         return *this;
     }
@@ -805,6 +975,9 @@ class basic_value
                 std::vector<std::string> com, region_type reg)
         : type_(value_t::offset_datetime), offset_datetime_(offset_datetime_storage(x, fmt)),
           region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(offset_datetime_type x)
     {
@@ -816,6 +989,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::offset_datetime;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->offset_datetime_, offset_datetime_storage(x, fmt));
         return *this;
     }
@@ -848,6 +1024,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::offset_datetime;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->offset_datetime_, offset_datetime_storage(offset_datetime_type(x), fmt));
         return *this;
     }
@@ -873,6 +1052,9 @@ class basic_value
         : type_(value_t::array), array_(array_storage(
               detail::storage<array_type>(std::move(x)), std::move(fmt)
           )), region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(array_type x)
     {
@@ -884,6 +1066,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::array;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->array_, array_storage(
                     detail::storage<array_type>(std::move(x)), std::move(fmt)));
         return *this;
@@ -893,8 +1078,14 @@ class basic_value
 
     template<typename T>
     using enable_if_array_like_t = cxx::enable_if_t<cxx::conjunction<
+            detail::is_container<T>,
             cxx::negation<std::is_same<T, array_type>>,
-            detail::is_container<T>
+            cxx::negation<detail::is_std_basic_string<T>>,
+#if defined(TOML11_HAS_STRING_VIEW)
+            cxx::negation<detail::is_std_basic_string_view<T>>,
+#endif
+            cxx::negation<detail::has_from_toml_method<T, config_type>>,
+            cxx::negation<detail::has_specialized_from<T>>
         >::value, std::nullptr_t>;
 
   public:
@@ -924,6 +1115,9 @@ class basic_value
                       std::make_move_iterator(x.end()))
               ), std::move(fmt)
           )), region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     template<typename T, enable_if_array_like_t<T> = nullptr>
     basic_value& operator=(T x)
@@ -936,7 +1130,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::array;
         this->region_ = region_type{};
-
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         array_type a(std::make_move_iterator(x.begin()),
                      std::make_move_iterator(x.end()));
         assigner(this->array_, array_storage(
@@ -965,6 +1161,9 @@ class basic_value
         : type_(value_t::table), table_(table_storage(
                 detail::storage<table_type>(std::move(x)), std::move(fmt)
           )), region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     basic_value& operator=(table_type x)
     {
@@ -976,6 +1175,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::table;
         this->region_ = region_type{};
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         assigner(this->table_, table_storage(
             detail::storage<table_type>(std::move(x)), std::move(fmt)));
         return *this;
@@ -988,7 +1190,9 @@ class basic_value
     template<typename T>
     using enable_if_table_like_t = cxx::enable_if_t<cxx::conjunction<
             cxx::negation<std::is_same<T, table_type>>,
-            detail::is_map<T>
+            detail::is_map<T>,
+            cxx::negation<detail::has_from_toml_method<T, config_type>>,
+            cxx::negation<detail::has_specialized_from<T>>
         >::value, std::nullptr_t>;
 
   public:
@@ -1018,6 +1222,9 @@ class basic_value
                       std::make_move_iterator(x.end())
               )), std::move(fmt)
           )), region_(std::move(reg)), comments_(std::move(com))
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
     template<typename T, enable_if_table_like_t<T> = nullptr>
     basic_value& operator=(T x)
@@ -1030,7 +1237,9 @@ class basic_value
         this->cleanup();
         this->type_   = value_t::table;
         this->region_ = region_type{};
-
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         table_type t(std::make_move_iterator(x.begin()),
                      std::make_move_iterator(x.end()));
         assigner(this->table_, table_storage(
@@ -1060,6 +1269,9 @@ class basic_value
     basic_value& operator=(const T& ud)
     {
         *this = into<cxx::remove_cvref_t<T>>::template into_toml<config_type>(ud);
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         return *this;
     }
 
@@ -1083,6 +1295,35 @@ class basic_value
     basic_value& operator=(const T& ud)
     {
         *this = ud.into_toml();
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
+        return *this;
+    }
+
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            detail::has_template_into_toml_method<T, TypeConfig>,
+            cxx::negation<detail::has_specialized_into<T>>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value(const T& ud): basic_value(ud.template into_toml<TypeConfig>()) {}
+
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            detail::has_template_into_toml_method<T, TypeConfig>,
+            cxx::negation<detail::has_specialized_into<T>>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value(const T& ud, std::vector<std::string> com)
+        : basic_value(ud.template into_toml<TypeConfig>(), std::move(com))
+    {}
+    template<typename T, cxx::enable_if_t<cxx::conjunction<
+            detail::has_template_into_toml_method<T, TypeConfig>,
+            cxx::negation<detail::has_specialized_into<T>>
+        >::value, std::nullptr_t> = nullptr>
+    basic_value& operator=(const T& ud)
+    {
+        *this = ud.template into_toml<TypeConfig>();
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         return *this;
     }
     // }}}
@@ -1092,6 +1333,9 @@ class basic_value
     // mainly for `null` extension
     basic_value(detail::none_t, region_type reg) noexcept
         : type_(value_t::empty), empty_('\0'), region_(std::move(reg)), comments_{}
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        , accessed_{false}
+#endif
     {}
 
     // }}}
@@ -1103,9 +1347,13 @@ class basic_value
         std::nullptr_t> = nullptr>
     bool is() const noexcept
     {
-        return detail::type_to_enum<T, value_type>::value == this->type_;
+        return this->is(detail::type_to_enum<T, value_type>::value);
+    }
+    bool is(value_t t) const noexcept
+    {
+        this->set_accessed();
+        return t == this->type_;
     }
-    bool is(value_t t) const noexcept {return t == this->type_;}
 
     bool is_empty()           const noexcept {return this->is(value_t::empty          );}
     bool is_boolean()         const noexcept {return this->is(value_t::boolean        );}
@@ -1121,6 +1369,7 @@ class basic_value
 
     bool is_array_of_tables() const noexcept
     {
+        this->set_accessed();
         if( ! this->is_array()) {return false;}
         const auto& a = this->as_array(std::nothrow); // already checked.
 
@@ -1140,7 +1389,11 @@ class basic_value
         return true;
     }
 
-    value_t type() const noexcept {return type_;}
+    value_t type() const noexcept
+    {
+        this->set_accessed();
+        return type_;
+    }
 
     // }}}
 
@@ -1150,36 +1403,38 @@ class basic_value
     detail::enum_to_type_t<T, basic_value<config_type>> const&
     as(const std::nothrow_t&) const noexcept
     {
+        this->set_accessed();
         return detail::getter<config_type, T>::get_nothrow(*this);
     }
     template<value_t T>
     detail::enum_to_type_t<T, basic_value<config_type>>&
     as(const std::nothrow_t&) noexcept
     {
+        this->set_accessed();
         return detail::getter<config_type, T>::get_nothrow(*this);
     }
 
-    boolean_type         const& as_boolean        (const std::nothrow_t&) const noexcept {return this->boolean_.value;}
-    integer_type         const& as_integer        (const std::nothrow_t&) const noexcept {return this->integer_.value;}
-    floating_type        const& as_floating       (const std::nothrow_t&) const noexcept {return this->floating_.value;}
-    string_type          const& as_string         (const std::nothrow_t&) const noexcept {return this->string_.value;}
-    offset_datetime_type const& as_offset_datetime(const std::nothrow_t&) const noexcept {return this->offset_datetime_.value;}
-    local_datetime_type  const& as_local_datetime (const std::nothrow_t&) const noexcept {return this->local_datetime_.value;}
-    local_date_type      const& as_local_date     (const std::nothrow_t&) const noexcept {return this->local_date_.value;}
-    local_time_type      const& as_local_time     (const std::nothrow_t&) const noexcept {return this->local_time_.value;}
-    array_type           const& as_array          (const std::nothrow_t&) const noexcept {return this->array_.value.get();}
-    table_type           const& as_table          (const std::nothrow_t&) const noexcept {return this->table_.value.get();}
+    boolean_type         const& as_boolean        (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->boolean_.value;}
+    integer_type         const& as_integer        (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->integer_.value;}
+    floating_type        const& as_floating       (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->floating_.value;}
+    string_type          const& as_string         (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->string_.value;}
+    offset_datetime_type const& as_offset_datetime(const std::nothrow_t&) const noexcept {this->set_accessed(); return this->offset_datetime_.value;}
+    local_datetime_type  const& as_local_datetime (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->local_datetime_.value;}
+    local_date_type      const& as_local_date     (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->local_date_.value;}
+    local_time_type      const& as_local_time     (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->local_time_.value;}
+    array_type           const& as_array          (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->array_.value.get();}
+    table_type           const& as_table          (const std::nothrow_t&) const noexcept {this->set_accessed(); return this->table_.value.get();}
 
-    boolean_type        & as_boolean        (const std::nothrow_t&) noexcept {return this->boolean_.value;}
-    integer_type        & as_integer        (const std::nothrow_t&) noexcept {return this->integer_.value;}
-    floating_type       & as_floating       (const std::nothrow_t&) noexcept {return this->floating_.value;}
-    string_type         & as_string         (const std::nothrow_t&) noexcept {return this->string_.value;}
-    offset_datetime_type& as_offset_datetime(const std::nothrow_t&) noexcept {return this->offset_datetime_.value;}
-    local_datetime_type & as_local_datetime (const std::nothrow_t&) noexcept {return this->local_datetime_.value;}
-    local_date_type     & as_local_date     (const std::nothrow_t&) noexcept {return this->local_date_.value;}
-    local_time_type     & as_local_time     (const std::nothrow_t&) noexcept {return this->local_time_.value;}
-    array_type          & as_array          (const std::nothrow_t&) noexcept {return this->array_.value.get();}
-    table_type          & as_table          (const std::nothrow_t&) noexcept {return this->table_.value.get();}
+    boolean_type        & as_boolean        (const std::nothrow_t&) noexcept {this->set_accessed(); return this->boolean_.value;}
+    integer_type        & as_integer        (const std::nothrow_t&) noexcept {this->set_accessed(); return this->integer_.value;}
+    floating_type       & as_floating       (const std::nothrow_t&) noexcept {this->set_accessed(); return this->floating_.value;}
+    string_type         & as_string         (const std::nothrow_t&) noexcept {this->set_accessed(); return this->string_.value;}
+    offset_datetime_type& as_offset_datetime(const std::nothrow_t&) noexcept {this->set_accessed(); return this->offset_datetime_.value;}
+    local_datetime_type & as_local_datetime (const std::nothrow_t&) noexcept {this->set_accessed(); return this->local_datetime_.value;}
+    local_date_type     & as_local_date     (const std::nothrow_t&) noexcept {this->set_accessed(); return this->local_date_.value;}
+    local_time_type     & as_local_time     (const std::nothrow_t&) noexcept {this->set_accessed(); return this->local_time_.value;}
+    array_type          & as_array          (const std::nothrow_t&) noexcept {this->set_accessed(); return this->array_.value.get();}
+    table_type          & as_table          (const std::nothrow_t&) noexcept {this->set_accessed(); return this->table_.value.get();}
 
     // }}}
 
@@ -1202,6 +1457,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_boolean()", value_t::boolean);
         }
+        this->set_accessed();
         return this->boolean_.value;
     }
     integer_type const& as_integer() const
@@ -1210,6 +1466,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_integer()", value_t::integer);
         }
+        this->set_accessed();
         return this->integer_.value;
     }
     floating_type const& as_floating() const
@@ -1218,6 +1475,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_floating()", value_t::floating);
         }
+        this->set_accessed();
         return this->floating_.value;
     }
     string_type const& as_string() const
@@ -1226,6 +1484,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_string()", value_t::string);
         }
+        this->set_accessed();
         return this->string_.value;
     }
     offset_datetime_type const& as_offset_datetime() const
@@ -1234,6 +1493,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_offset_datetime()", value_t::offset_datetime);
         }
+        this->set_accessed();
         return this->offset_datetime_.value;
     }
     local_datetime_type const& as_local_datetime() const
@@ -1242,6 +1502,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_local_datetime()", value_t::local_datetime);
         }
+        this->set_accessed();
         return this->local_datetime_.value;
     }
     local_date_type const& as_local_date() const
@@ -1250,6 +1511,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_local_date()", value_t::local_date);
         }
+        this->set_accessed();
         return this->local_date_.value;
     }
     local_time_type const& as_local_time() const
@@ -1258,6 +1520,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_local_time()", value_t::local_time);
         }
+        this->set_accessed();
         return this->local_time_.value;
     }
     array_type const& as_array() const
@@ -1266,6 +1529,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_array()", value_t::array);
         }
+        this->set_accessed();
         return this->array_.value.get();
     }
     table_type const& as_table() const
@@ -1274,6 +1538,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_table()", value_t::table);
         }
+        this->set_accessed();
         return this->table_.value.get();
     }
 
@@ -1286,6 +1551,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_boolean()", value_t::boolean);
         }
+        this->set_accessed();
         return this->boolean_.value;
     }
     integer_type& as_integer()
@@ -1294,6 +1560,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_integer()", value_t::integer);
         }
+        this->set_accessed();
         return this->integer_.value;
     }
     floating_type& as_floating()
@@ -1302,6 +1569,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_floating()", value_t::floating);
         }
+        this->set_accessed();
         return this->floating_.value;
     }
     string_type& as_string()
@@ -1310,6 +1578,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_string()", value_t::string);
         }
+        this->set_accessed();
         return this->string_.value;
     }
     offset_datetime_type& as_offset_datetime()
@@ -1318,6 +1587,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_offset_datetime()", value_t::offset_datetime);
         }
+        this->set_accessed();
         return this->offset_datetime_.value;
     }
     local_datetime_type& as_local_datetime()
@@ -1326,6 +1596,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_local_datetime()", value_t::local_datetime);
         }
+        this->set_accessed();
         return this->local_datetime_.value;
     }
     local_date_type& as_local_date()
@@ -1334,6 +1605,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_local_date()", value_t::local_date);
         }
+        this->set_accessed();
         return this->local_date_.value;
     }
     local_time_type& as_local_time()
@@ -1342,6 +1614,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_local_time()", value_t::local_time);
         }
+        this->set_accessed();
         return this->local_time_.value;
     }
     array_type& as_array()
@@ -1350,6 +1623,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_array()", value_t::array);
         }
+        this->set_accessed();
         return this->array_.value.get();
     }
     table_type& as_table()
@@ -1358,6 +1632,7 @@ class basic_value
         {
             this->throw_bad_cast("toml::value::as_table()", value_t::table);
         }
+        this->set_accessed();
         return this->table_.value.get();
     }
 
@@ -1767,10 +2042,22 @@ class basic_value
     comment_type const& comments() const noexcept {return this->comments_;}
     comment_type&       comments()       noexcept {return this->comments_;}
 
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+    bool accessed() const {return this->accessed_.load();}
+#endif
+
   private:
 
     // private helper functions =========================================== {{{
 
+    void set_accessed() const noexcept
+    {
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_.store(true);
+#endif
+        return;
+    }
+
     void cleanup() noexcept
     {
         switch(this->type_)
@@ -1787,6 +2074,9 @@ class basic_value
             case value_t::table           : { table_          .~table_storage           (); break; }
             default                       : { break; }
         }
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+        this->accessed_ = false;
+#endif
         this->type_ = value_t::empty;
         return;
     }
@@ -1819,6 +2109,12 @@ class basic_value
     template<typename TC>
     friend class basic_value;
 
+
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+    template<typename TC>
+    friend void detail::unset_access_flag(basic_value<TC>&);
+#endif
+
     // }}}
 
   private:
@@ -1853,6 +2149,10 @@ class basic_value
     };
     region_type  region_;
     comment_type comments_;
+
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+    mutable std::atomic<bool> accessed_;
+#endif
 };
 
 template<typename TC>
@@ -2062,10 +2362,10 @@ error_info make_type_error(const basic_value<TC>& v, const std::string& fname, c
         v.location(), "the actual type is " + to_string(v.type()));
 }
 template<typename TC>
-error_info make_not_found_error(const basic_value<TC>& v, const std::string& fname, const std::string& key)
+error_info make_not_found_error(const basic_value<TC>& v, const std::string& fname, const typename basic_value<TC>::key_type& key)
 {
     const auto loc = v.location();
-    const std::string title = fname + ": key \"" + key + "\" not found";
+    const std::string title = fname + ": key \"" + string_conv<std::string>(key) + "\" not found";
 
     std::vector<std::pair<source_location, std::string>> locs;
     if( ! loc.is_ok())
@@ -2163,6 +2463,48 @@ void change_region_of_value(basic_value<TC>& dst, const basic_value<TC>& src)
     return;
 }
 
+#ifdef TOML11_ENABLE_ACCESS_CHECK
+template<typename TC>
+void unset_access_flag(basic_value<TC>& v)
+{
+    v.accessed_.store(false);
+}
+
+template<typename TC>
+void unset_access_flag_recursively(basic_value<TC>& v)
+{
+    switch(v.type())
+    {
+        case value_t::empty            : { return unset_access_flag(v); }
+        case value_t::boolean          : { return unset_access_flag(v); }
+        case value_t::integer          : { return unset_access_flag(v); }
+        case value_t::floating         : { return unset_access_flag(v); }
+        case value_t::string           : { return unset_access_flag(v); }
+        case value_t::offset_datetime  : { return unset_access_flag(v); }
+        case value_t::local_datetime   : { return unset_access_flag(v); }
+        case value_t::local_date       : { return unset_access_flag(v); }
+        case value_t::local_time       : { return unset_access_flag(v); }
+        case value_t::array:
+        {
+            for(auto& elem : v.as_array())
+            {
+                unset_access_flag_recursively(elem);
+            }
+            return unset_access_flag(v);
+        }
+        case value_t::table:
+        {
+            for(auto& kv : v.as_table())
+            {
+                unset_access_flag_recursively(kv.second);
+            }
+            return unset_access_flag(v);
+        }
+        default: { return unset_access_flag(v); }
+    }
+}
+#endif
+
 } // namespace detail
 } // namespace toml
 #endif // TOML11_VALUE_HPP
